==> python-cupy-rocm-gfx9-13.6.0-10-x86_64.pkg.tar.zst: usr/lib/python3.14/site-packages/cupy/_core/include/cupy/_cccl/thrust/thrust/iterator/detail/tagged_iterator.h <==
/*
 *  Copyright 2008-2013 NVIDIA Corporation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

#pragma once

#include <thrust/detail/config.h>

#if defined(_CCCL_IMPLICIT_SYSTEM_HEADER_GCC)
#  pragma GCC system_header
#elif defined(_CCCL_IMPLICIT_SYSTEM_HEADER_CLANG)
#  pragma clang system_header
#elif defined(_CCCL_IMPLICIT_SYSTEM_HEADER_MSVC)
#  pragma system_header
#endif // no system header
#include <thrust/detail/use_default.h>
#include <thrust/iterator/iterator_adaptor.h>
#include <thrust/iterator/iterator_traits.h>
#include <thrust/type_traits/is_contiguous_iterator.h>

THRUST_NAMESPACE_BEGIN
namespace detail
{

template <typename, typename>
class tagged_iterator;

template <typename Iterator, typename Tag>
struct tagged_iterator_base
{
  using type =
    thrust::iterator_adaptor<tagged_iterator<Iterator, Tag>,
                             Iterator,
                             typename thrust::iterator_value<Iterator>::type,
                             Tag,
                             typename thrust::iterator_traversal<Iterator>::type,
                             typename thrust::iterator_reference<Iterator>::type,
                             typename thrust::iterator_difference<Iterator>::type>;
}; // end tagged_iterator_base

template <typename Iterator, typename Tag>
class tagged_iterator : public tagged_iterator_base<Iterator, Tag>::type
{
private:
  using super_t = typename tagged_iterator_base<Iterator, Tag>::type;

public:
  _CCCL_HOST_DEVICE tagged_iterator() {}

  _CCCL_HOST_DEVICE explicit tagged_iterator(Iterator x)
      : super_t(x)
  {}
}; // end tagged_iterator

/*! \p make_tagged_iterator creates a \p tagged_iterator
 *  from a \c Iterator with system tag \c Tag.
 *
 *  \tparam Tag Any system tag.
 *  \tparam Iterator Any iterator type.
 *  \param iter The iterator of interest.
 *  \return An iterator whose system tag is \p Tag and whose behavior is otherwise
 *          equivalent to \p iter.
 */
template <typename Tag, typename Iterator>
inline auto make_tagged_iterator(Iterator iter) -> tagged_iterator<Iterator, Tag>
{
  return tagged_iterator<Iterator, Tag>(iter);
}

} // namespace detail

// tagged_iterator is trivial if its base iterator is.
template <typename BaseIterator, typename Tag>
struct proclaim_contiguous_iterator<detail::tagged_iterator<BaseIterator, Tag>> : is_contiguous_iterator<BaseIterator>
{};

THRUST_NAMESPACE_END

==> python-cupy-rocm-gfx9-14.0.1-2-x86_64.pkg.tar.zst: usr/lib/python3.14/site-packages/cupy/_core/include/cupy/_cccl/thrust/thrust/iterator/detail/tagged_iterator.h <==
/*
 *  Copyright 2008-2013 NVIDIA Corporation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

#pragma once

#include <thrust/detail/config.h>

#if defined(_CCCL_IMPLICIT_SYSTEM_HEADER_GCC)
#  pragma GCC system_header
#elif defined(_CCCL_IMPLICIT_SYSTEM_HEADER_CLANG)
#  pragma clang system_header
#elif defined(_CCCL_IMPLICIT_SYSTEM_HEADER_MSVC)
#  pragma system_header
#endif // no system header
#include <thrust/detail/use_default.h>
#include <thrust/iterator/iterator_adaptor.h>
#include <thrust/iterator/iterator_traits.h>
#include <thrust/type_traits/is_contiguous_iterator.h>

THRUST_NAMESPACE_BEGIN
namespace detail
{
template <typename, typename>
class tagged_iterator;

template <typename Iterator, typename Tag>
using make_tagged_iterator_base =
  iterator_adaptor<tagged_iterator<Iterator, Tag>,
                   Iterator,
                   it_value_t<Iterator>,
                   Tag,
                   typename iterator_traversal<Iterator>::type,
                   it_reference_t<Iterator>,
                   it_difference_t<Iterator>>;

template <typename Iterator, typename Tag>
class tagged_iterator : public make_tagged_iterator_base<Iterator, Tag>
{
  using super_t = make_tagged_iterator_base<Iterator, Tag>;

public:
  tagged_iterator() = default;

  _CCCL_HOST_DEVICE explicit tagged_iterator(Iterator x)
      : super_t(x)
  {}
};

//! \p make_tagged_iterator creates a \p tagged_iterator from a \c Iterator with system tag \c Tag.
//!
//! \tparam Tag Any system tag.
//! \tparam Iterator Any iterator type.
//! \param iter The iterator of interest.
//! \return An iterator whose system tag is \p Tag and whose behavior is otherwise equivalent to \p iter.
template <typename Tag, typename Iterator>
auto make_tagged_iterator(Iterator iter) -> tagged_iterator<Iterator, Tag>
{
  return tagged_iterator<Iterator, Tag>(iter);
}

} // namespace detail

// tagged_iterator is trivial if its base iterator is.
template <typename BaseIterator, typename Tag>
struct proclaim_contiguous_iterator<detail::tagged_iterator<BaseIterator, Tag>> : is_contiguous_iterator<BaseIterator>
{};

THRUST_NAMESPACE_END
